==> imapfilter_1.0.1.orig.tar.gz: imapfilter-1.0.1/core.c <==
#include <stdio.h>

#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>

#include "imapfilter.h"


static int ifcore_login(lua_State * lua);
static int ifcore_logout(lua_State * lua);
static int ifcore_status(lua_State * lua);
static int ifcore_select(lua_State * lua);
static int ifcore_close(lua_State * lua);
static int ifcore_search(lua_State * lua);
static int ifcore_fetch(lua_State * lua);
static int ifcore_store(lua_State * lua);
static int ifcore_copy(lua_State * lua);
static int ifcore_append(lua_State * lua);


/* Lua imapfilter core library functions. */
static const luaL_reg ifcorelib[] = {
	{"login", ifcore_login},
	{"logout", ifcore_logout},
	{"status", ifcore_status},
	{"select", ifcore_select},
	{"close", ifcore_close},
	{"search", ifcore_search},
	{"fetch", ifcore_fetch},
	{"store", ifcore_store},
	{"copy", ifcore_copy},
	{"append", ifcore_append},
	{NULL, NULL}
};


/*
 * Core function to login to the server.
 */
static int
ifcore_login(lua_State * lua)
{
	const char *s, *u, *p;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	if (!(p = get_table_string("password")))
		luaL_error(lua, "no password specified");

	r = request_login(s, u, p, (unsigned int)(get_table_number("port")),
	    get_table_string("ssl"));

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == RESPONSE_OK || r == RESPONSE_PREAUTH));

	return 1;
}


/*
 * Core function to logout from the server.
 */
static int
ifcore_logout(lua_State * lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");

	r = request_logout(s, u);

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	return 1;
}


/*
 * Core function to get the status of a mailbox.
 */
static int
ifcore_status(lua_State * lua)
{
	const char *s, *u;
	int r;
	int exists, recent, unseen;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_status(s, u, lua_tostring(lua, 2), &exists, &recent, &unseen);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == RESPONSE_OK));
	lua_pushnumber(lua, (lua_Number) (exists));
	lua_pushnumber(lua, (lua_Number) (recent));
	lua_pushnumber(lua, (lua_Number) (unseen));

	return 4;
}


/*
 * Core function to select a mailbox.
 */
static int
ifcore_select(lua_State * lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_select(s, u, lua_tostring(lua, 2));

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	return 1;
}


/*
 * Core function to close a mailbox.
 */
static int
ifcore_close(lua_State * lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");

	r = request_close(s, u);

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	return 1;
}


/*
 * Core function to search the messages of a mailbox.
 */
static int
ifcore_search(lua_State * lua)
{
	char *m;
	const char *s, *u;
	int r;

	m = NULL;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_search(s, u, lua_tostring(lua, 2), &m);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	if (!m)
		return 1;

	lua_pushstring(lua, m);

	xfree(m);

	return 2;
}


/*
 * Core function to fetch message items.
 */
static int
ifcore_fetch(lua_State * lua)
{
	const char *s, *u;
	int r;
	char *f;

	if (lua_gettop(lua) != 3)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_fetch(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3), &f);

	lua_pop(lua, 3);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	if (!f)
		return 1;

	lua_pushstring(lua, f);

	return 2;
}


/*
 * Core function to change message flags.
 */
static int
ifcore_store(lua_State * lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 4)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);
	luaL_checktype(lua, 4, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_store(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
	    lua_tostring(lua, 4));

	lua_pop(lua, 4);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	return 1;
}


/*
 * Core function to copy messages between mailboxes.
 */
static int
ifcore_copy(lua_State * lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 3)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_copy(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3));

	lua_pop(lua, 3);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	return 1;
}


/*
 * Core function to append messages to a mailbox.
 */
static int
ifcore_append(lua_State * lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 5)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);
	luaL_checktype(lua, 4, LUA_TSTRING);
	luaL_checktype(lua, 5, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_append(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
	    lua_tostring(lua, 4), lua_tostring(lua, 5));

	lua_pop(lua, 5);

	lua_pushboolean(lua, (r == RESPONSE_OK));

	return 1;
}


/*
 * Open imapfilter core library.
 */
LUALIB_API int
luaopen_ifcore(lua_State * lua)
{

	luaL_openlib(lua, "ifcore", ifcorelib, 0);
	return 1;
}

==> imapfilter_1.2.1.orig.tar.gz: imapfilter-1.2.1/core.c <==
#include <stdio.h>

#include <lua.h>
#include <lauxlib.h>
#include <lualib.h>

#include "imapfilter.h"


static int ifcore_noop(lua_State *lua);
static int ifcore_login(lua_State *lua);
static int ifcore_logout(lua_State *lua);
static int ifcore_status(lua_State *lua);
static int ifcore_select(lua_State *lua);
static int ifcore_close(lua_State *lua);
static int ifcore_search(lua_State *lua);
static int ifcore_list(lua_State *lua);
static int ifcore_lsub(lua_State *lua);
static int ifcore_fetchfast(lua_State *lua);
static int ifcore_fetchheader(lua_State *lua);
static int ifcore_fetchtext(lua_State *lua);
static int ifcore_fetchfields(lua_State *lua);
static int ifcore_store(lua_State *lua);
static int ifcore_copy(lua_State *lua);
static int ifcore_append(lua_State *lua);


/* Lua imapfilter core library functions. */
static const luaL_reg ifcorelib[] = {
	{ "noop", ifcore_noop },
	{ "login", ifcore_login },
	{ "logout", ifcore_logout },
	{ "status", ifcore_status },
	{ "select", ifcore_select },
	{ "close", ifcore_close },
	{ "list", ifcore_list },
	{ "lsub", ifcore_lsub },
	{ "search", ifcore_search },
	{ "fetchfast", ifcore_fetchfast },

	/*
	 * RFC 822: message == header + body
	 * RFC 3501: body == header + text
	 * 
	 * RFC 3501 notation is used internally, and RFC 822 notation is used
	 * for the interface available to the user.
	 */
	{ "fetchheader", ifcore_fetchheader },
	{ "fetchbody", ifcore_fetchtext }, 

	{ "fetchfields", ifcore_fetchfields },
	{ "store", ifcore_store },
	{ "copy", ifcore_copy },
	{ "append", ifcore_append },
	{ NULL, NULL }
};


/*
 * Core function to reset any inactivity autologout timer on the server.
 */
static int
ifcore_noop(lua_State *lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");

	r = request_noop(s, u);

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Core function to login to the server.
 */
static int
ifcore_login(lua_State *lua)
{
	const char *s, *u, *p;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	if (!(p = get_table_string("password")))
		luaL_error(lua, "no password specified");

	r = request_login(s, u, p, get_table_string("port"),
	    get_table_string("ssl"));

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK ||
	    r == STATUS_RESPONSE_PREAUTH));

	return 1;
}


/*
 * Core function to logout from the server.
 */
static int
ifcore_logout(lua_State *lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");

	r = request_logout(s, u);

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Core function to get the status of a mailbox.
 */
static int
ifcore_status(lua_State *lua)
{
	const char *s, *u;
	int r;
	unsigned int exists, recent, unseen;

	exists = recent = unseen = 0;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_status(s, u, lua_tostring(lua, 2), &exists, &recent,
	    &unseen);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));
	lua_pushnumber(lua, (lua_Number) (exists));
	lua_pushnumber(lua, (lua_Number) (recent));
	lua_pushnumber(lua, (lua_Number) (unseen));

	return 4;
}


/*
 * Core function to select a mailbox.
 */
static int
ifcore_select(lua_State *lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_select(s, u, lua_tostring(lua, 2));

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Core function to close a mailbox.
 */
static int
ifcore_close(lua_State *lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 1)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);

	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");

	r = request_close(s, u);

	lua_pop(lua, 1);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Core function to list available mailboxes.
 */
static int
ifcore_list(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *mboxs, *folders;

	mboxs = folders = NULL;

	if (lua_gettop(lua) != 3)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_list(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
	    &mboxs, &folders);

	lua_pop(lua, 3);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!mboxs && !folders)
		return 1;

	lua_pushstring(lua, mboxs);
	lua_pushstring(lua, folders);

	xfree(mboxs);
	xfree(folders);

	return 3;
}


/*
 * Core function to list subscribed mailboxes.
 */
static int
ifcore_lsub(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *mboxs, *folders;

	mboxs = folders = NULL;

	if (lua_gettop(lua) != 3)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_lsub(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
	    &mboxs, &folders);

	lua_pop(lua, 3);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!mboxs)
		return 1;

	lua_pushstring(lua, mboxs);
	lua_pushstring(lua, folders);

	xfree(mboxs);

	return 3;
}


/*
 * Core function to search the messages of a mailbox.
 */
static int
ifcore_search(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *mesgs;

	mesgs = NULL;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_search(s, u, lua_tostring(lua, 2), &mesgs);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!mesgs)
		return 1;

	lua_pushstring(lua, mesgs);

	xfree(mesgs);

	return 2;
}


/*
 * Core function to fetch message information (flags, date, size).
 */
static int
ifcore_fetchfast(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *flags, *date, *size;

	flags = date = size = NULL;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_fetchfast(s, u, lua_tostring(lua, 2), &flags, &date, &size);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!flags || !date || !size)
		return 1;

	lua_pushstring(lua, flags);
	lua_pushstring(lua, date);
	lua_pushstring(lua, size);

	xfree(flags);
	xfree(date);
	xfree(size);

	return 4;
}


/*
 * Core function to fetch message header.
 */
static int
ifcore_fetchheader(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *header;
	size_t len;

	header = NULL;
	len = 0;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_fetchheader(s, u, lua_tostring(lua, 2), &header, &len);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!header)
		return 1;

	lua_pushlstring(lua, header, len);

	return 2;
}


/*
 * Core function to fetch message text.
 */
static int
ifcore_fetchtext(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *text;
	size_t len;

	text = NULL;
	len = 0;

	if (lua_gettop(lua) != 2)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_fetchtext(s, u, lua_tostring(lua, 2), &text, &len);

	lua_pop(lua, 2);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!text)
		return 1;

	lua_pushlstring(lua, text, len);

	return 2;
}


/*
 * Core function to fetch message specific header fields.
 */
static int
ifcore_fetchfields(lua_State *lua)
{
	const char *s, *u;
	int r;
	char *fields;
	size_t len;

	fields = NULL;
	len = 0;

	if (lua_gettop(lua) != 3)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_fetchfields(s, u, lua_tostring(lua, 2),
	    lua_tostring(lua, 3), &fields, &len);

	lua_pop(lua, 3);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	if (!fields)
		return 1;

	lua_pushlstring(lua, fields, len);

	return 2;
}


/*
 * Core function to change message flags.
 */
static int
ifcore_store(lua_State *lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 4)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);
	luaL_checktype(lua, 4, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_store(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
	    lua_tostring(lua, 4));

	lua_pop(lua, 4);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Core function to copy messages between mailboxes.
 */
static int
ifcore_copy(lua_State *lua)
{
	const char *s, *u;
	int r;

	if (lua_gettop(lua) != 3)
		luaL_error(lua, "wrong number of arguments");

	luaL_checktype(lua, 1, LUA_TTABLE);
	luaL_checktype(lua, 2, LUA_TSTRING);
	luaL_checktype(lua, 3, LUA_TSTRING);

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_copy(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3));

	lua_pop(lua, 3);

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Core function to append messages to a mailbox.
 */
static int
ifcore_append(lua_State *lua)
{
	const char *s, *u;
	int r;

	switch (lua_gettop(lua)) {
	case 5:
		luaL_checktype(lua, 5, LUA_TSTRING);
		/* FALLTHROUGH */
	case 4:
		luaL_checktype(lua, 4, LUA_TSTRING);
		/* FALLTHROUGH */
	case 3:
		luaL_checktype(lua, 3, LUA_TSTRING);
		luaL_checktype(lua, 2, LUA_TSTRING);
		luaL_checktype(lua, 1, LUA_TTABLE);
		break;
	default:
		luaL_error(lua, "wrong number of arguments");
		break;
	}

	lua_pushvalue(lua, 1);
	if (!(s = get_table_string("server")))
		luaL_error(lua, "no mail server specified");
	if (!(u = get_table_string("username")))
		luaL_error(lua, "no username specified");
	lua_pop(lua, 1);

	r = request_append(s, u, lua_tostring(lua, 2), lua_tostring(lua, 3),
	    lua_strlen(lua, 3), lua_tostring(lua, 4), lua_tostring(lua, 5));

	lua_pop(lua, lua_gettop(lua));

	lua_pushboolean(lua, (r == STATUS_RESPONSE_OK));

	return 1;
}


/*
 * Open imapfilter core library.
 */
LUALIB_API int
luaopen_ifcore(lua_State *lua)
{

	luaL_openlib(lua, "ifcore", ifcorelib, 0);

	return 1;
}
